@@ -20,14 +20,14 @@ pub(crate) fn mark_unchanged<'a>(
 ) -> Vec<(Vec<&'a Syntax<'a>>, Vec<&'a Syntax<'a>>)> {
     let (_, lhs_nodes, rhs_nodes) = shrink_unchanged_at_ends(lhs_nodes, rhs_nodes, change_map);
 
-    let mut res = vec![];
+    let mut nodes_to_diff = vec![];
     for (lhs_nodes, rhs_nodes) in split_mostly_unchanged_toplevel(&lhs_nodes, &rhs_nodes) {
         let (_, lhs_nodes, rhs_nodes) =
             shrink_unchanged_at_ends(&lhs_nodes, &rhs_nodes, change_map);
-        res.extend(split_unchanged(&lhs_nodes, &rhs_nodes, change_map));
+        nodes_to_diff.extend(split_unchanged(&lhs_nodes, &rhs_nodes, change_map));
     }
 
-    res
+    nodes_to_diff
 }
 
 #[derive(Debug)]
