@@ -19,6 +19,9 @@
 // implementation does not consider the mutable fields, so it is still
 // correct.
 #![allow(clippy::mutable_key_type)]
+// It's sometimes more readable to explicitly create a vec than to use
+// the Default trait.
+#![allow(clippy::manual_unwrap_or_default)]
 // .to_owned() is more explicit on string references.
 #![warn(clippy::str_to_string)]
 // .to_string() on a String is clearer as .clone().
@@ -373,16 +376,21 @@ fn diff_file(
         rhs_src.retain(|c| c != '\r');
     }
 
-    // If "foo" is one line, is "foo\n" two lines? Generally we want
-    // to care about newlines when deciding whether content differs.
+    // Ensure that lhs_src and rhs_src both have trailing
+    // newlines.
     //
-    // Ending a file with a trailing newline is extremely common
-    // though. If both files have a trailing newline, consider "foo\n"
-    // to be "foo" so we don't end up displaying a blank line on both
-    // sides.
-    if lhs_src.ends_with('\n') && rhs_src.ends_with('\n') {
-        lhs_src.pop();
-        rhs_src.pop();
+    // This is important when textually diffing files that don't have
+    // a trailing newline, e.g. "foo\n\bar\n" versus "foo". We want to
+    // consider `foo` to be unchanged in this case.
+    //
+    // Theoretically a tree-sitter parser coud change its AST due to
+    // the additional trailing newline, but it seems vanishingly
+    // unlikely.
+    if !lhs_src.is_empty() && !lhs_src.ends_with('\n') {
+        lhs_src.push('\n');
+    }
+    if !rhs_src.is_empty() && !rhs_src.ends_with('\n') {
+        rhs_src.push('\n');
     }
 
     let mut extra_info = renamed;
@@ -733,7 +741,7 @@ fn diff_file_content(
 /// incrementally.
 ///
 /// When more than one file is modified, the hg extdiff extension passes directory
-/// paths with the all the modified files.
+/// paths with all the modified files.
 fn diff_directories<'a>(
     lhs_dir: &'a Path,
     rhs_dir: &'a Path,
