@@ -94,7 +94,8 @@ impl Default for DiffOptions {
 fn app() -> clap::Command<'static> {
     Command::new("Difftastic")
         .override_usage(USAGE)
-        .version(VERSION.as_str())
+        .version(env!("CARGO_PKG_VERSION"))
+        .long_version(VERSION.as_str())
         .about(crate_description!())
         .author(crate_authors!())
         .after_long_help(concat!(
@@ -260,7 +261,7 @@ When multiple overrides are specified, the first matching override wins."))
         )
         .arg(
             Arg::new("list-languages").long("list-languages")
-                .help("Print the all the languages supported by difftastic, along with their extensions.")
+                .help("Print all the languages supported by difftastic, along with their extensions.")
         )
         .arg(
             Arg::new("byte-limit").long("byte-limit")
@@ -778,10 +779,11 @@ pub(crate) fn parse_args() -> Mode {
                 None,
             )
         }
-        [old_name, lhs_tmp_file, _lhs_hash, lhs_mode, rhs_tmp_file, _rhs_hash, rhs_mode, new_name, _similarity] =>
+        [old_name, lhs_tmp_file, _lhs_hash, lhs_mode, rhs_tmp_file, _rhs_hash, rhs_mode, new_name, _metainfo] =>
         {
             // Rename file.
             // TODO: where does git document these 9 arguments?
+            // (See run_external_diff() in diff.c in git source code.)
 
             let old_name = old_name.to_string_lossy().to_string();
             let new_name = new_name.to_string_lossy().to_string();
